@@ -8,7 +8,7 @@
  *
  * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
  *
- * Copyright (C) 2003-2021 by Thomas Dreibholz
+ * Copyright (C) 2003-2022 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@
 #include <sys/socket.h>
 #include <inttypes.h>
 #include <stdio.h>
-#include <ext_socket.h>
 
+#include "ext_socket.h"
 #include "tdtypes.h"
 #include "sockaddrunion.h"
 
